﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.Threading.Tasks;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime;
using System.Runtime.Serialization;

namespace Example4Lib
{
    [ServiceContract]
    public class CMatrix
    {
        [OperationContract]
        public Stream Generate(int size, int min, int max)
        {
            Random rnd = new Random();
            float[][] matrix = new float[size][];
            for (int i = 0; i < size; i++)
            {
                matrix[i] = new float[size];
                for (int j = 0; j < size; j++)
                {
                    matrix[i][j] = (float)(rnd.Next(min * 100, (max + 1) * 100) / 100.0);
                }
            }
            return BinSerializer.Serialize(matrix);            
        }
        [OperationContract]
        public Stream Multiple(Stream streamIn)
        {
            CMatrixData data = (CMatrixData)BinSerializer.Deserialize(streamIn);
            float[][] m1, m2;
            m1 = data.matrixOne;
            m2 = data.matrixTwo;
            int size = m1.GetLength(0);
            float[][] res = new float[size][];
            for (int i = 0; i < size; i++)
            {
                res[i] = new float[size];
                for (int j = 0; j < size; j++)
                    res[i][j] = 0;
            }
            Parallel.For(data.beginInteration, data.endIteration, delegate(int i)
            {
                for (int j = 0; j < m1.GetLength(0); j++)
                    for (int k = 0; k < m1.GetLength(0); k++)
                        res[i][j] += m1[i][k] * m2[k][j];
            });
            return BinSerializer.Serialize(res);
        }
    }
    public class BinSerializer
    {
        public static MemoryStream Serialize(object obj)
        {
            MemoryStream stream = new MemoryStream();
            BinaryFormatter formatter = new BinaryFormatter();
            formatter.Serialize(stream, obj);
            stream.Seek(0, SeekOrigin.Begin);
            return stream;
        }
        public static object Deserialize(Stream s)
        {
            BinaryFormatter formatterIn = new BinaryFormatter();
            return formatterIn.Deserialize(s);
        }
    }
    
    [DataContract, Serializable]
    public struct CMatrixData
    {
        public float[][] matrixOne, matrixTwo;
        public int beginInteration, endIteration;
    }
}